-- Filename:     counter_integer.vhd
-- Filetype:     VHDL Design File
-- Date:         9 mar 2014
-- Update:       -
-- Description:  A behavioral description of a counter using integers
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2014, De Haagse Hogeschool

-- The design file contains the description of a integer
-- up counter with a load and a asynchronous reset. The 
-- counter provides a terminal count signal for signalling
-- that the counter has reached its maximum value.

-- Use library std_logic_1164
-- Use library numeric_std for conversion

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- The entity, default an 8 bit unsigned counter
entity counter_integer is
	generic (n   : positive := 8);
	port (clk    : in std_logic;
			areset : in std_logic;
			load   : in std_logic;
			data   : in unsigned(n-1 downto 0);
			q      : out unsigned(n-1 downto 0);
			tc     : out std_logic
	);
end entity counter_integer;

-- The architecture
architecture rtl of counter_integer is
-- Range is 0 to 2 to-the-power-of-n minus 1
signal count_val : integer range 0 to 2**n-1;
begin
	-- Process is sensitive to ...
	process (clk, areset) is
	begin
		-- Asynchronous reset, active high
		if areset = '1' then
			count_val <= 0;
		elsif rising_edge(clk) then
		   -- Load is active, use conversion routine
			if load = '1' then
				count_val <= to_integer(data);
			else
			   -- We have to count. Please note the explicit
				-- test to 2**n-1, which is mandatory for simulation,
				-- but (in this case) not for synthesis, because 2**n-1
				-- are all 1's in the counter bits.
				if count_val = 2**n-1 then
					count_val <= 0;
				else
					count_val <= count_val + 1;
				end if;
			end if;
		end if;
	end process;

	-- Conversion of integer to unsigned using the length.
	q <= to_unsigned(count_val, q'length);
	
	-- The terminal count
	tc <= '1' when count_val = 2**n-1 else '0';
	
end architecture rtl;
